/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.readGraph.getAssertedStatements;

import java.util.Collection;
import java.util.UUID;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Statement;
import org.simantics.db.WriteGraph;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.testing.base.WriteReadTest;
import org.simantics.db.testing.common.Tests;
import org.simantics.db.tests.common.Writes;

public class GetAssertedStatementsTest5 extends WriteReadTest {

    private Resource type;
    private Resource type2;
    private Resource type3;
    private Resource named1;
    private Resource named2;
    private Resource named3;
    private Resource named4;
    private Resource named5;
    private Resource named6;

	@Override
	protected void write(WriteGraph graph) throws DatabaseException {

        type2 = graph.newResource();
        graph.claim(type2, L0.Inherits, L0.Entity);
        type3 = graph.newResource();
        graph.claim(type3, L0.Inherits, L0.Entity);
        type = graph.newResource();
        graph.claim(type, L0.Inherits, type2);
        graph.claim(type, L0.Inherits, type3);
        
        named1 = Writes.named(graph, UUID.randomUUID().toString());
        named2 = Writes.named(graph, UUID.randomUUID().toString());
        named3 = Writes.named(graph, UUID.randomUUID().toString());
        named4 = Writes.named(graph, UUID.randomUUID().toString());
        named5 = Writes.named(graph, UUID.randomUUID().toString());
        named6 = Writes.named(graph, UUID.randomUUID().toString());

        graph.claim(type, L0.Asserts, Writes.assertion(graph, L0.ConsistsOf, named1));
        graph.claim(type, L0.Asserts, Writes.assertion(graph, L0.ConsistsOf, named2));
        graph.claim(type2, L0.Asserts, Writes.assertion(graph, L0.ConsistsOf, named3));
        graph.claim(type2, L0.Asserts, Writes.assertion(graph, L0.ConsistsOf, named4));
        graph.claim(type3, L0.Asserts, Writes.assertion(graph, L0.ConsistsOf, named5));
        graph.claim(type3, L0.Asserts, Writes.assertion(graph, L0.ConsistsOf, named6));
		
	}
	
	@Override
	protected void read(ReadGraph graph) throws DatabaseException {
        Collection<Statement> results = graph.getAssertedStatements(type, L0.ConsistsOf);
        assertNotNull("Null not allowed", results);
        assertTrue(Tests.contains(results, L0.ConsistsOf, named1));
        assertTrue(Tests.contains(results, L0.ConsistsOf, named2));
        assertTrue(Tests.contains(results, L0.ConsistsOf, named3));
        assertTrue(Tests.contains(results, L0.ConsistsOf, named4));
        assertTrue(Tests.contains(results, L0.ConsistsOf, named5));
        assertTrue(Tests.contains(results, L0.ConsistsOf, named6));
        assertTrue(results.size() == 6);
	}
}
