/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.readGraph.getSingleType;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.exception.NoSingleResultException;
import org.simantics.db.testing.base.WriteReadTest;
import org.simantics.db.tests.common.Writes;

public class GetSingleTypeTest3 extends WriteReadTest {

    private Resource type1;
    private Resource type2;
    private Resource type3;
    private Resource named1;
    private Resource instance1;
    
	@Override
	protected void write(WriteGraph graph) throws DatabaseException {

        type1 = graph.newResource();
        graph.claim(type1, L0.Inherits, L0.Entity);
        type2 = graph.newResource();
        graph.claim(type2, L0.Inherits, L0.Entity);
        type3 = graph.newResource();
        graph.claim(type3, L0.Inherits, L0.Entity);


        graph.claim(type3, L0.Asserts, Writes.assertion(graph, L0.Inherits, type1));
        graph.claim(type3, L0.Asserts, Writes.assertion(graph, L0.Inherits, type2));

        instance1 = graph.newResource();
        graph.claim(instance1, L0.InstanceOf, type3);
	}
	
	@Override
    protected void read(ReadGraph graph) throws DatabaseException {
    	try {
    		Resource result = graph.getSingleType(instance1);
    		assertNotNull("Null not allowed", result);
    		assertTrue("Wrong type", result.equals(type3));
    		return;
    	} catch (NoSingleResultException e) {
    		fail("More than single results");
    	} catch (Throwable t) {
    		fail("Wrong exception: " + t.getClass().getCanonicalName());
    	}
    	fail("Should throw");
    }
    
}
