/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.misc;

import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.ConcurrentSkipListSet;

import org.junit.Test;
import org.simantics.db.AsyncReadGraph;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.procedure.AsyncProcedure;
import org.simantics.db.procedure.Procedure;
import org.simantics.db.request.AsyncRead;
import org.simantics.db.request.Read;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;

public class SyncAsyncSyncTest extends ExistingDatabaseTest {
	
	@Test
    public void test() throws Exception {
		
		Session session = getSession();
		
		final ConcurrentSkipListSet<String> names = new ConcurrentSkipListSet<String>();
		final ArrayList<Resource> resources = new ArrayList<Resource>();
		final Resource root = SyncAsyncSyncUtils.createNames(session, names, resources);

		session.syncRequest(new TestReadRequest() {

            @Override
            public void run(ReadGraph graph) throws Throwable {
                
                final Layer0 b = Layer0.getInstance(graph);
                Collection<Resource> rs = graph.getObjects(root, b.ConsistsOf);
                for(final Resource r : rs) {

                    graph.asyncRequest(new AsyncRead<String>() {

                        @Override
                        public void perform(AsyncReadGraph graph, AsyncProcedure<String> procedure) {
                            
                            graph.asyncRequest(new Read<String>() {

                                @Override
                                public String perform(ReadGraph graph) throws DatabaseException {
                                    return graph.getRelatedValue(r, b.HasName);
                                }
                                
                            }, procedure);
                            
                        }

                        @Override
            		    public int threadHash() {
            		    	return hashCode();
            		    }

                        @Override
                        public int getFlags() {
                            return 0;
                        }
      
                    }, new Procedure<String>() {

                        @Override
                        public void exception(Throwable t) {
                            t.printStackTrace();
                        }

                        @Override
                        public void execute(String result) {
                            names.remove(result);
                        }
                        
                    });
                    
                }
                
            }
		    
		});
		
		assert(names.isEmpty());
		
	}

}
