/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.support.clusterControl;

import java.util.Collection;

import org.junit.Test;
import org.simantics.databoard.Bindings;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.db.WriteOnlyGraph;
import org.simantics.db.common.request.WriteOnlyResultRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.ClusterControl;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.db.testing.common.WriteQuery;
import org.simantics.db.tests.common.Configuration;
import org.simantics.layer0.Layer0;

/**
 * Creates large amount of instances in multiple transactions
 * and the tries to verify that instances are written into the database.
 * 
 * @author Marko Luukkainen <marko.luukkainen@vtt.fi>
 *
 */
public class WriteAfterClusterCollectTest2 extends ExistingDatabaseTest {
	private int LOOP = Configuration.get().collectedLoopCount;
	@Test
	public void test() throws Exception{

	    final String random = getRandomString();
		
		final Resource rootLib = getSession().syncRequest(new WriteOnlyResultRequest<Resource>() {
		    @Override
		    public Resource perform(WriteOnlyGraph g) throws DatabaseException {
		    	Resource rootLib = g.newResource();
                for (int i = 0; i < 10; i++) {
    		        for (int j = 0; j < LOOP; j++) {
    		            Layer0 b = Layer0.getInstance(getSession());
    		            String newInstanceName = random + i;
    		            Resource instance = g.newResource();
    		            g.claim(instance, b.InstanceOf, null, b.Type);
    		            g.addLiteral(instance, b.HasName, b.NameOf, b.String, newInstanceName, Bindings.STRING);
    		            g.claim(rootLib, b.ConsistsOf, b.PartOf, instance);
    		            g.flushCluster();
    		        }
                }
                return rootLib;
		    }
		});
		
		checkException();
		
		getSession().syncRequest(new TestReadRequest() {
			@Override
			public void run(ReadGraph g) throws Throwable {
				Layer0 b = Layer0.getInstance(g);
				Collection<Resource> resources = g.getObjects(rootLib, b.ConsistsOf);
				for (Resource r : resources) {
					String name = g.getPossibleRelatedValue(r, b.HasName);
					if (DEBUG)
					    System.out.println("Name is " + name);
				}
			}
		});
		
		ClusterControl support = getSession().getService(ClusterControl.class);
		support.collectClusters(Integer.MAX_VALUE);
		
		checkException();
		
        getSession().syncRequest(new WriteQuery(this) {
            @Override
            public void run(WriteGraph g) throws Throwable {
                
                Layer0 b = Layer0.getInstance(g);
                Collection<Resource> resources = g.getObjects(rootLib, b.ConsistsOf);
                for (Resource r : resources) {
                    String name = g.getPossibleRelatedValue(r, b.HasName);
                    if (DEBUG)
                        System.out.println("Name is " + name);
                    String newInstanceName = random;
                    Resource instance = g.newResource();
                    g.claimLiteral(instance, b.HasName, newInstanceName);
                    g.claim(r, b.ConsistsOf, b.PartOf, instance);
                }
                
            }
            
        });
        
        checkException();

        getSession().syncRequest(new TestReadRequest() {
            @Override
            public void run(ReadGraph g) throws Throwable {
                Layer0 b = Layer0.getInstance(g);
                Collection<Resource> resources = g.getObjects(rootLib, b.ConsistsOf);
                for (Resource r : resources) {
                    String name = g.getPossibleRelatedValue(r, b.HasName);
                    if (DEBUG)
                        System.out.println("Name is " + name);
                }
            }
        });
        
        checkException();

        support.collectClusters(Integer.MAX_VALUE);
		
	}

}
