/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.support.undoRedoSupport;

import java.util.Arrays;

import org.junit.Test;
import org.simantics.databoard.Bindings;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.UndoRedoSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;
import org.simantics.utils.DataContainer;

/**
 * Test case description missing!
 */
public class UndoTest16 extends ExistingDatabaseTest {
    private void initValue(byte[] value) {
        for (int i=0; i<value.length; ++i)
            value[i] = (byte)i;
    }
	@Test
    public void testUndo16() throws Exception {

    	Session session = getSession();
        final UndoRedoSupport support = session.getService(UndoRedoSupport.class);
        final DataContainer<Resource> subject = new DataContainer<Resource>();
        final byte[] value = new byte[100000];
        final byte[] newValue = new byte[200000];
        
        try {
            initValue(value);
            initValue(newValue);
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph graph) throws DatabaseException {
                    graph.markUndoPoint();
                    // Use any non-functional relation
                    Layer0 b = Layer0.getInstance(graph);
                    Resource s = graph.newResource();
                    graph.claim(s, b.InstanceOf, b.Entity);
                    graph.claimValue(s, value);
                    subject.set(s);
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }

        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                    Resource s = subject.get();
                    if (!graph.hasStatement(s))
                        fail("Failed to verify modifications (stm).");
                    byte[] t = graph.getValue(s, Bindings.BYTE_ARRAY);
                    if (!Arrays.equals(t, value))
                        fail("Failed to verify modifications (value).");
                }
            });
        } catch (Throwable e) {
            fail("Read transaction threw an unexpected exception " + e);
        }

        try {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph graph) throws DatabaseException {
                    graph.markUndoPoint();
                    Resource s = subject.get();
                    graph.claimValue(s, newValue);
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }

        support.undo(session, 1);

        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                    Resource s = subject.get();
                    byte[] t = graph.getValue(s, Bindings.BYTE_ARRAY);
                    if (!Arrays.equals(t, value))
                        fail("Failed to verify modifications (value).");
                }
            });
        } catch (Throwable e) {
            fail("Read transaction threw an unexpected exception " + e);
        }

        support.undo(session, 1);

        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                    Resource s = subject.get();
                    byte[] t = graph.getPossibleValue(s, Bindings.BYTE_ARRAY);
                    if (null != t)
                        fail("Failed to verify modifications (value).");
                }
            });
        } catch (Throwable e) {
            fail("Read transaction threw an unexpected exception " + e);
        }
    }
}
