/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.client;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.exception.ValidationException;
import org.simantics.db.service.LifecycleSupport;
import org.simantics.db.testing.annotation.Fails;
import org.simantics.db.testing.common.TestBase;
import org.simantics.db.testing.common.Tests;
import org.simantics.db.testing.impl.Configuration;
import org.simantics.layer0.Layer0;

public class ConnectionTest5 extends TestBase {
	private static final int RECONNECT_COUNT = Configuration.get().connectionReconnectCount;
    private static final int SAVE_COUNT = Configuration.get().connectionSaveCount;
    private static final int INSTANCE_COUNT = Configuration.get().connectionInstanceCount;
	private static final boolean SKIP = Configuration.get().skipServerCreation;
	private Session session = null;
	private final String random = "ConnectionTest5" + TestBase.getRandomString();
    private final String libraryName = "Library" + random;

    @Test
    @Fails
	public void test5() throws DatabaseException {
	    if (SKIP) {
	    	System.out.println("This test is not valid for remote server.");
	    	return;
	    }
        // Try to load and register the Driver instance with the driver manager
        try {
            Class.forName("fi.vtt.simantics.procore.ProCoreDriver");
        } catch (ClassNotFoundException e) {
            throw new DatabaseException("Test failed.", e);
        }
        for (int i=0; i<RECONNECT_COUNT; ++i) {
            connectionTest();
        }
	}
    void connectionTest() throws DatabaseException {
        initTest();
        List<String> names = new ArrayList<String>();
        for (int i=0; i<SAVE_COUNT; ++i) {
             String name = saveTest(i+1);
             names.add(name);
        }
        checkTest(names);
    }
    void initTest() throws DatabaseException {
        try {
            session = Tests.getTestHandler().getSession();
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph g) throws DatabaseException {
                    Layer0 b = Layer0.getInstance(g);
                    Resource newResource = g.newResource();
                    g.claim(newResource, b.InstanceOf, null, b.Library);
                    g.claimLiteral(newResource, b.HasName, libraryName);
                    g.claim(g.getRootLibrary(), b.ConsistsOf, newResource);
                }
            });

            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph g) throws DatabaseException {
                    Layer0 l0 = Layer0.getInstance(g);
                    Resource rl = g.getResource(TestBase.ROOT_LIBRARY_URI);
                    Collection<Resource> resources = g.getObjects(rl, l0.ConsistsOf);
                    Resource newResource = null;
                    for (Resource r : resources) {
                        String value = g.getPossibleRelatedValue(r, l0.HasName);
                        if (null != value && value.equals(libraryName)) {
                            newResource = r;
                            break;
                        }
                    }
                    if (newResource == null) {
                        throw new ValidationException("Could not find created resource");
                    }
                }
            });
            LifecycleSupport ls = session.getService(LifecycleSupport.class);
            ls.close();
        } finally {
            Tests.closeSession(session);
        }
	}
    private Resource getResourceByNameAndRelation(ReadGraph g,
            Resource parent, Resource relation, String name)
    throws DatabaseException {
        Collection<Resource> resources = g.getObjects(parent, relation);
        Resource newResource = null;
        Layer0 l0 = Layer0.getInstance(g);
        for (Resource r : resources) {
            String value = g.getPossibleRelatedValue(r, l0.HasName);
            if (null != value && value.equals(name)) {
                newResource = r;
                break;
            }
        }
        return newResource;
    }
    String saveTest(int id) throws DatabaseException {
        try {
            session = Tests.getTestHandler().getSession();
            final String subLibraryName = "" + id + random;
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph g) throws DatabaseException {
                    Layer0 b = Layer0.getInstance(g);
                    Resource subLibrary = g.newResource();
                    g.claim(subLibrary, b.InstanceOf, null, b.Library);
                    g.claimLiteral(subLibrary, b.HasName, subLibraryName);
                    Resource library = getResourceByNameAndRelation(g,
                            g.getRootLibrary(), b.ConsistsOf, libraryName);
                    assertTrue(null != library);
                    g.claim(library, b.ConsistsOf, b.PartOf, subLibrary);
                    for (int j=0; j<INSTANCE_COUNT; ++j) {
                        Resource instance = g.newResource();
                        g.claim(instance, b.InstanceOf, null, b.Library);
                        String instanceName = ""+(j+1);
                        g.claimLiteral(instance, b.HasName, instanceName);
                        g.claim(subLibrary, b.ConsistsOf, b.PartOf, instance);
                    }
                }
            });

            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph g) throws DatabaseException {
                    Layer0 l0 = Layer0.getInstance(g);
                    Resource rl = g.getResource(TestBase.ROOT_LIBRARY_URI);
                    Resource library = getResourceByNameAndRelation(g,
                            rl, l0.ConsistsOf, libraryName);
                    assertTrue(null != library);
                    Resource subLibrary = getResourceByNameAndRelation(g,
                            library, l0.ConsistsOf, subLibraryName);
                    assertTrue(null != subLibrary);
                    for (int j=0; j<INSTANCE_COUNT; ++j) {
                        String name = ""+(j+1);
                        Resource instance = getResourceByNameAndRelation(g,
                                subLibrary, l0.ConsistsOf, name);
                        assertTrue(instance != null);
                    }
                }
            });
            LifecycleSupport ls = session.getService(LifecycleSupport.class);
            ls.close();
            return subLibraryName;
        } finally {
            Tests.closeSession(session);
        }
    }
    void checkTest(final List<String> names) throws DatabaseException {
        try {
            session = Tests.getTestHandler().getSession();
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph g) throws DatabaseException {
                    Layer0 l0 = Layer0.getInstance(g);
                    Resource rl = g.getResource(TestBase.ROOT_LIBRARY_URI);
                    Resource library = getResourceByNameAndRelation(g,
                            rl, l0.ConsistsOf, libraryName);
                    assertTrue(null != library);
                    for (String libraryName : names) {
                        Resource subLibrary = getResourceByNameAndRelation(g,
                                library, l0.ConsistsOf, libraryName);
                        assertTrue(null != subLibrary);
                        for (int j=0; j<INSTANCE_COUNT; ++j) {
                            String name = ""+(j+1);
                            Resource instance = getResourceByNameAndRelation(g,
                                    subLibrary, l0.ConsistsOf, name);
                            assertTrue(instance != null);
                        }
                    }
                }
            });
            LifecycleSupport ls = session.getService(LifecycleSupport.class);
            ls.close();
//        } catch (ClassNotFoundException e) {
//            throw new DatabaseException("Test failed.", e);
        } finally {
            Tests.closeSession(session);
        }
    }
}
