/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.scalability;

import java.util.Collection;
import java.util.UUID;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.db.testing.common.WriteQuery;
import org.simantics.layer0.Layer0;

public class LargeCommitTest extends ExistingDatabaseTest {
    
    private static final int OUTER_INSTANCE_COUNT = 10; // these go to the root library
    private static final int INNER_INSTANCE_COUNT = 100; // these are for each outer instance

    /**
     * Creates new type and new instance of it. Then tries to find the instance and the type, and verifies that inheritance is correct. 
     * @throws Exception
     */
    @Test
    public void testCommit() throws Exception{
        Session session = getSession();
        final Resource rootLib = getSession().getRootLibrary();
        final String random = UUID.randomUUID().toString();
        final String newTypeName = "New Type" + random; 
        session.syncRequest(new WriteQuery(this) {
			@Override
			public void run(WriteGraph g) throws Throwable {
                Layer0 b = Layer0.getInstance(g);
				Resource newType = g.newResource();
                g.claim(newType, b.InstanceOf, null, b.Type);
                g.claim(newType, b.Inherits, b.SupertypeOf, b.Type);
                g.claimLiteral(newType, b.HasName, newTypeName);
                g.claim(rootLib, b.ConsistsOf, newType);
                
                for (int i=0; i<OUTER_INSTANCE_COUNT; ++i) {
                    //System.out.println("Write outer=" + i);
                    Resource outerInstance = g.newResource();
                    g.claim(outerInstance, b.InstanceOf, null, newType);
                    String outerInstanceName = random+(i+1);
                    g.claimLiteral(outerInstance, b.HasName, outerInstanceName);
                    g.claim(rootLib, b.ConsistsOf, outerInstance);
                    for (int j=0; j<INNER_INSTANCE_COUNT; ++j) {
                        Resource innerInstance = g.newResource();
                        g.claim(innerInstance, b.InstanceOf, null, newType);
                        String innerInstanceName = ""+(j+1);
                        g.claimLiteral(innerInstance, b.HasName, innerInstanceName);
                        g.claim(outerInstance, b.ConsistsOf, innerInstance);
                    }
                }
            }
        });
        
        checkException();
        
        session.syncRequest(new TestReadRequest() {
            @Override
            public void run(ReadGraph g) throws Exception  {
                Layer0 b = Layer0.getInstance(g);
                Collection<Resource> resources = g.getObjects(rootLib, b.ConsistsOf);
                Resource newType = null;
                Resource newInstance[] = new Resource[OUTER_INSTANCE_COUNT];
                for (Resource r : resources) {
                    String name = g.getPossibleRelatedValue(r, b.HasName);
                    if (null == name)
                    	continue;
                    if (name.equals(newTypeName)) {
                        newType = r;
                    }
                    if (!name.startsWith(random, 0))
                        continue;
                    name = name.substring(random.length());
                    int index = 0;
                    try {
                        index = Integer.parseInt(name);
                    } catch (NumberFormatException e) {
                        continue; // not a number
                    }
                    if (index > 0 && index < OUTER_INSTANCE_COUNT+1) {
                        newInstance[index-1] = r;
                    }
                }
                if (newType == null) 
                    throw new Exception("Could not find created type");
                for (int i=0; i<OUTER_INSTANCE_COUNT; ++i) {
                    //System.out.println("Read outer=" + i);
                    if (newInstance[i] == null) 
                        throw new Exception("Could not find created instance");
                    if (!g.isInstanceOf(newInstance[i], b.Type))
                        throw new Exception("Created resource is not an instance of Type");
                    if (!g.isInstanceOf(newInstance[i], newType))
                        throw new Exception("Created resource is not an instance of created type");
                    // TODO: check inner instances
                }
            }
        });
        
        checkException();
        
    }

}
