package org.simantics.scl.compiler.tests;

import org.junit.Assert;
import org.junit.Test;
import org.simantics.scl.compiler.elaboration.java.Builtins;
import org.simantics.scl.compiler.environment.specification.EnvironmentSpecification;
import org.simantics.scl.compiler.module.ImportDeclaration;
import org.simantics.scl.compiler.module.repository.ModuleRepository;
import org.simantics.scl.compiler.module.repository.UpdateListener;
import org.simantics.scl.compiler.runtime.RuntimeEnvironment;
import org.simantics.scl.compiler.source.PrecompiledModuleSource;
import org.simantics.scl.compiler.source.StringModuleSource;
import org.simantics.scl.compiler.source.repository.MapModuleSourceRepository;
import org.simantics.scl.compiler.source.repository.ModuleSourceRepository;
import org.simantics.scl.compiler.top.ExpressionEvaluator;

public class TestClassNaming {

    private static class SimpleModuleSource extends StringModuleSource {
        public SimpleModuleSource(String moduleName, String moduleText) {
            super(moduleName, moduleText);
        }
        
        @Override
        protected ImportDeclaration[] getBuiltinImports(UpdateListener listener) {
            return new ImportDeclaration[] {new ImportDeclaration("Builtin", "")};
        }
    }
    
    @Test
    public void testClassNaming() throws Exception {
        ModuleSourceRepository sourceRepository = new MapModuleSourceRepository(
                new PrecompiledModuleSource(Builtins.INSTANCE),
                new SimpleModuleSource("http://ProjectGame@A/SCLConstants",
                        "locale = \"fi-FI\"")
                );
        ModuleRepository moduleRepository = new ModuleRepository(sourceRepository);
        RuntimeEnvironment runtimeEnvironment = moduleRepository.createRuntimeEnvironment(
                EnvironmentSpecification.of(
                        "http://ProjectGame@A/SCLConstants", ""),
                getClass().getClassLoader());
        Assert.assertEquals("fi-FI", new ExpressionEvaluator(runtimeEnvironment, "locale").eval());
    }
    
}
